/* File: ArgumentPassing.c
// A simple C program that calls an subroutine defined external to this source module.
//
*/

#include <avr/io.h>
#include <avr/interrupt.h>

int add(int x, int y ); // declaration for a function defined locally in this source file


// extern tells the linker that this variable is DEFINED in some other source file
// The "extern" is REQUIRED in this case
extern int asm_counter;      // defined in sub1.s
extern int c_counter;        // defined in subroutine.c
extern int private_counter;  // defined in subroutine.c as "static", so it's private to that module and cannot be accessed here, although it can be declared here without error

// extern tells the linker this function is DEFINED in some other source file
// The "extern" keyword is OPTIONAL (but recommended) in this case
extern uint16_t asm_add(uint16_t x, uint16_t y); // an externally defined assembly function defined in sub1.s
extern int fun1();                               // an externally defined C function defined in subroutine.c

// These are global variables available to any function
// int: 2 bytes each, whether initialized or not
 int w;
 int x;
 int z;
 char msg[]="Hello"; // 6 bytes including null terminator
 
 // volatile means the compiler will always get the latest value from memory when this variable is manipulated
 volatile int y=3;   // always declare global variables volatile if they get changed in ISRs.
 
// #define  LCDPORTA (*(uint8_t*)(0x38))

int main()			/* every C program needs a "main" */
{

    // __asm let's us insert assembly code at any point
    __asm("nop"); // this is just a nop, so it's not dangerous

    fun1(); // call the function defined in subroutine.c

// local variables are limited in scope to "main"
    uint16_t temp = asm_counter; // use the asm_counter variable defined in sub1.s

    temp = asm_add(20000,15000);   // call the C-compatible assembly function defined in the assuembly source file

    y = add(1,3);   // call the C function defined below
    asm_counter++;  // asm_counter is defined in sub1.s
    c_counter++;    // defined in subroutine.c
//    private_counter++; // defined in subroutine.c, but declared "static" (which makes it private to that module), so it cannot be accessed here
}

//ISR Macro used to indicate to GCC that this is an ISR
ISR(INT1_vect) {
    PORTB = 0xFF;
    y = 5;      // change the value of the global volatile variable
}



// local function definition
int add(int x, int y ) {
   return x + y;
}
