// An assembly source module containing several subroutines.
//
// .global makes the symbol "sub1" visible (callable) outside this source module,
//   similar to declaring a method "public" in Java
.global sub1

// without .global, the function is "private" to this source module
.global asm_add


// Subroutine sub1. Globally visible (callable) from outside. 
// Just calls sub2
sub1:
	rcall sub2
	ret

// Subroutine sub2. Locally visible; callable only within this source module (similar to "private" in C++ or Java)
// Just calls sub3
sub2:
	rcall sub3
	ret

// Subroutine sub3. Locally visible if not declared ".global" above
sub3: 
    ret


//  this C-compatible subroutine adds two 16-bit values together and returns their sum
//  arg1 in r25:r24 (low, high bytes), and arg2 in r23:r22
asm_add:
    add r24, r22    ; add the low bytes
    adc r25, r23    ; add high bytes with carry bit included

	ret             ; return the sum in r25:r24 (low, high)

// data section. defines a global 2-byte value visible from C as the variable "counter"
.section .data
.global asm_counter

// here's the actual definition of the "variable"; it's really just a labeled location in SRAM, but
// looks like a variable to a C program
asm_counter: .word 0x1234
